@@ -1,6 +1,7 @@
 import pytest
 from rpy2 import rinterface
 from rpy2.rinterface import embedded
+from rpy2.rinterface_lib import sexp
 
 
 @pytest.mark.skipif(embedded.rpy2_embeddedR_isinitialized,
@@ -21,5 +22,9 @@ def test_set_initoptions():
 def test_assert_isready():
     with pytest.raises(embedded.RNotReadyError):
         embedded.assert_isready()
-        
-    
+
+@pytest.mark.skipif(embedded.rpy2_embeddedR_isinitialized,
+                    reason='Can only be tested before R is initialized.')
+def test_assert_environment_geitem():
+    with pytest.raises(embedded.RNotReadyError):
+        sexp.globalenv['x']
